// MicrosoftStack.cpp: implementation of the CMicrosoftStack class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MicrosoftStack.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


#define	OUTBUFSIZE	16384

CMicrosoftStack::CMicrosoftStack()
:	m_hThread		(NULL),
	m_dwThreadID	(0),
	m_bStopThread	(false),
	m_hWnd			(NULL),
	m_nMsg			(0)
{
	WSAStartup(0x0202, &m_wsaData);

	m_vecSdpRec.reserve(32);

	CoInitializeEx(NULL, COINIT_MULTITHREADED);
}


CMicrosoftStack::~CMicrosoftStack()
{
	ClearDiscovery();
	CoUninitialize();
	WSACleanup();
}


BOOL CMicrosoftStack::StartInquiry()
{
	if(m_hThread != NULL)
	{
		if(WaitForSingleObject(m_hThread, 2000) == WAIT_TIMEOUT)
			return FALSE;

		CloseHandle(m_hThread);
		m_hThread = NULL;
	}

	m_hThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)FindDevicesThread, this, 0, &m_dwThreadID);
	return m_hThread != NULL;
}


void CMicrosoftStack::StopInquiry()
{
	m_bStopThread = true;

	WaitForSingleObject(m_hThread, INFINITE);

	m_bStopThread = false;
}


// CMicrosoftStack::StartDiscovery
//
//		Starts the service discovery on a device
//
BOOL CMicrosoftStack::StartDiscovery(BTH_ADDR btAddr, GUID *pGuid)
{
	if(m_hThread != NULL)
	{
		StopInquiry();
		CloseHandle(m_hThread);
		m_hThread = NULL;
	}

	m_btAddr = btAddr;

	memcpy(&m_guid, pGuid, sizeof(GUID));
	
	m_hThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)DiscoverServicesThread, this, 0, &m_dwThreadID);
	return m_hThread != NULL;
}


DWORD CMicrosoftStack::DiscoverServicesThread(void *pVoid)
{
	CMicrosoftStack* pStack = (CMicrosoftStack*)pVoid;

	if(pStack)
		return pStack->DiscoverServices(pStack->m_btAddr, &pStack->m_guid);
	return 1;
}


// CMicrosoftStack::DiscoverServices
//
//		Enumerates the services in the remote device
//
DWORD CMicrosoftStack::DiscoverServices(BTH_ADDR btAddr, GUID *pGuid)
{
	HANDLE					hLookup;
	DWORD					dwFlags		= 0,
							dwLen;
	BYTE*					pOut		= NULL;
	WSAQUERYSET*			pQuerySet;
	CSADDR_INFO				csi;
	SOCKADDR_BTH			sa;
	TCHAR					szServiceName[256];
	int						nRet		= 0;
	int						nRecords	= 0;
	WPARAM					wParam		= S_OK;
	
	BLOB					blob;
	BTHNS_RESTRICTIONBLOB	resBlob;
	HRESULT					hr;

	// Allocate the output buffer
	pOut = (BYTE*)LocalAlloc(LPTR, OUTBUFSIZE);
	if(!pOut)
	{
		wParam = E_OUTOFMEMORY;
		goto error_exit;
	}


	pQuerySet = (WSAQUERYSET*)pOut;

	memset(&sa, 0, sizeof(sa));
	memset(&csi, 0, sizeof(csi));

	sa.btAddr			= btAddr;
	sa.addressFamily	= AF_BT;

	// Specify the remote device address
	csi.RemoteAddr.lpSockaddr		= (LPSOCKADDR)&sa;
	csi.RemoteAddr.iSockaddrLength	= sizeof(SOCKADDR_BTH);

	// Setup the restriction BLOB
	resBlob.type					= SDP_SERVICE_SEARCH_ATTRIBUTE_REQUEST;
	resBlob.uuids[0].uuidType		= SDP_ST_UUID128;
//	resBlob.uuids[0].u.uuid128		= *pGuid;
	resBlob.numRange				= 1;
	resBlob.pRange[0].minAttribute	= 0;		//SDP_ATTRIB_PROTOCOL_DESCRIPTOR_LIST;
	resBlob.pRange[0].maxAttribute	= 0xffff;	//SDP_ATTRIB_PROTOCOL_DESCRIPTOR_LIST;

	memcpy(&resBlob.uuids[0].u.uuid128, pGuid, sizeof(GUID));

	blob.cbSize		= sizeof(resBlob);
	blob.pBlobData	= (BYTE*)&resBlob;

	memset(pQuerySet, 0, OUTBUFSIZE);

	pQuerySet->dwSize				= sizeof(WSAQUERYSET);
	pQuerySet->dwNameSpace			= NS_BTH;
//	pQuerySet->lpServiceClassId		= pGuid;

	pQuerySet->lpcsaBuffer			= &csi;
	pQuerySet->dwNumberOfCsAddrs	= 1;

	pQuerySet->lpBlob				= &blob;

	ClearDiscovery();

	dwFlags	= 0;
	nRet = WSALookupServiceBegin(pQuerySet, dwFlags, &hLookup);
	if(nRet == SOCKET_ERROR)
	{
		wParam = WSAGetLastError();
		goto error_exit;
	}

	// Setup query set for ServiceNext call
	dwFlags	= LUP_RETURN_NAME | LUP_RETURN_ADDR;
	dwLen	= OUTBUFSIZE;

	pQuerySet->lpszServiceInstanceName	= szServiceName;
	memset(szServiceName, 0, sizeof(szServiceName));

	while((nRet = WSALookupServiceNext(hLookup, dwFlags, &dwLen, pQuerySet)) == 0)
	{
		ISdpRecord**	pRecordArg;
		int				cRecordArg	= 0;

		hr = ParseBlobsToRecs(pQuerySet->lpBlob->pBlobData,
							  pQuerySet->lpBlob->cbSize,
							  &pRecordArg,
							  (ULONG*)&cRecordArg);

		if(SUCCEEDED(hr))
		{
			int i;

			// Add all the Sdp records to the discovery object
			for(i = 0; i < cRecordArg; ++i)
			{
				CSdpRecord*	pSdpRecord	= new CSdpRecord(pRecordArg[i]);

				m_vecSdpRec.push_back(pSdpRecord);
				++nRecords;
			}
			CoTaskMemFree(pRecordArg);
		}
		else
		{
			wParam = hr;
			goto error_exit;
		}

		dwLen	= OUTBUFSIZE;
		pQuerySet->dwNumberOfCsAddrs = 1;
		pQuerySet->lpszServiceInstanceName = szServiceName;
		memset(szServiceName, 0, sizeof(szServiceName));
	}

	// Check if the enumeration finished correctly.
	wParam = WSAGetLastError();
	if(wParam == WSAENOMORE || wParam == WSA_E_NO_MORE)
		wParam = S_OK;

	// Cleanup
	WSALookupServiceEnd(hLookup);

error_exit:
	if(pOut)
		LocalFree(pOut);

	if(m_hWnd != NULL)
	{
		::SendMessage(m_hWnd, m_nMsg, wParam, nRecords);
	}

	return nRet;
}


HRESULT	CMicrosoftStack::ParseBlobsToRecs(UCHAR *pbData, DWORD cbStream, ISdpRecord ***pppSdpRecords, ULONG *pcbRec)
{
	HRESULT		hr;
	ULONG		ulError;
	ISdpStream*	pIStream	= NULL;
	
	*pppSdpRecords	= NULL;
	*pcbRec			= 0;

	hr = CoCreateInstance(__uuidof(SdpStream), NULL, CLSCTX_INPROC_SERVER, __uuidof(ISdpStream), (LPVOID*)&pIStream);
	if(FAILED(hr))
		return hr;

	hr = pIStream->Validate(pbData, cbStream, &ulError);
	if(SUCCEEDED(hr))
	{
		hr = pIStream->VerifySequenceOf(pbData, cbStream, SDP_TYPE_SEQUENCE, NULL, pcbRec);

		if(SUCCEEDED(hr) && *pcbRec > 0)
		{
			*pppSdpRecords = (ISdpRecord **)CoTaskMemAlloc(sizeof(ISdpRecord*) * (*pcbRec));

			if(*pppSdpRecords != NULL)
			{
				hr = pIStream->RetrieveRecords(pbData, cbStream, *pppSdpRecords, pcbRec);

				if(FAILED(hr))
				{
					CoTaskMemFree(*pppSdpRecords);
					*pppSdpRecords	= NULL;
					*pcbRec			= 0;
				}
			}
			else
				hr = E_OUTOFMEMORY;
		}
	}

	if(pIStream != NULL)
	{
		pIStream->Release();
		pIStream = NULL;
	}

	return hr;
}


DWORD CMicrosoftStack::FindDevices()
{
	BTHNS_INQUIRYBLOB	inqBlob;
	BLOB				blob;
	WSAQUERYSET			wsaQuerySet;
	WSAQUERYSET*		pQuerySet;
	HANDLE				hLookup;
	int					nRet;
	int					i;
	BYTE*				pOut;

	// Allocate the output buffer
	pOut = (BYTE*)LocalAlloc(LPTR, OUTBUFSIZE);
	if(!pOut)
		return 1;

	pQuerySet = (WSAQUERYSET*)pOut;

	memset(&inqBlob, 0, sizeof(inqBlob));

	inqBlob.LAP				= BT_ADDR_GIAC;
	inqBlob.length			= 4;				// 4 * 1.28 = 5 seconds
	inqBlob.num_responses	= 255;

	blob.cbSize		= sizeof(BTHNS_INQUIRYBLOB);
	blob.pBlobData	= (BYTE*)&inqBlob;

	//
	// Initialize the query
	//
	memset(&wsaQuerySet, 0, sizeof(wsaQuerySet));

	wsaQuerySet.dwSize		= sizeof(WSAQUERYSET);
	wsaQuerySet.dwNameSpace	= NS_BTH;
	wsaQuerySet.lpBlob		= &blob;

	// Start looking for devices
	nRet = WSALookupServiceBegin(&wsaQuerySet, LUP_CONTAINERS, &hLookup);

	for(i = 0; i < 255; ++i)
	{
		DWORD	dwLen	= OUTBUFSIZE;
		DWORD	dwFlags	= LUP_RETURN_NAME | LUP_RETURN_ADDR | LUP_RETURN_BLOB;

		if(m_bStopThread)
			break;

		nRet = WSALookupServiceNext(hLookup, dwFlags, &dwLen, pQuerySet);
		if(nRet == SOCKET_ERROR)
		{
			nRet = GetLastError();
			break;
		}

		if(m_hWnd != NULL)
		{
			BLUETOOTH_DEVICE_INFO*	pDevInfo = (BLUETOOTH_DEVICE_INFO*)LocalAlloc(LPTR, sizeof(BLUETOOTH_DEVICE_INFO));

			if(pDevInfo)
			{
				SOCKADDR_BTH		*pBtAddr	= (SOCKADDR_BTH*)		pQuerySet->lpcsaBuffer->RemoteAddr.lpSockaddr;
				BthInquiryResult	*pBtInq		= (BthInquiryResult*)	pQuerySet->lpBlob->pBlobData;

				pDevInfo->dwSize			= sizeof(BLUETOOTH_DEVICE_INFO);
				pDevInfo->Address.ullLong	= pBtAddr->btAddr;
				pDevInfo->ulClassofDevice	= pBtInq->cod;
				
				wcscpy(pDevInfo->szName, pQuerySet->lpszServiceInstanceName);

				SendMessage(m_hWnd, m_nMsg, 0, (LPARAM)pDevInfo);
			}
		}

	}

	if(nRet == WSA_E_NO_MORE)
		nRet = 0;

	// Cleanup
	WSALookupServiceEnd(hLookup);
	LocalFree(pOut);

	if(m_hWnd != NULL)
		::SendMessage(m_hWnd, m_nMsg, 1, MAKELPARAM(nRet, i));

	return 0;
}


DWORD CMicrosoftStack::FindDevicesThread(void *pVoid)
{
	CMicrosoftStack* pStack = (CMicrosoftStack*)pVoid;

	if(pStack)
		return pStack->FindDevices();
	return 1;
}


void CMicrosoftStack::ClearDiscovery()
{
	size_t	i;

	for(i = 0; i < m_vecSdpRec.size(); ++i)
	{
		delete m_vecSdpRec[i];
		m_vecSdpRec[i] = NULL;
	}
	m_vecSdpRec.clear();
}
