// NodeData.cpp: implementation of the CNodeData class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "NodeData.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


CNodeData::CNodeData()
{
	type = SDP_TYPE_NIL;
}


CNodeData::~CNodeData()
{
	switch(type)
	{
	case SDP_TYPE_STRING:
		CoTaskMemFree(u.str.val);
		break;

	case SDP_TYPE_URL:
		CoTaskMemFree(u.url.val);
		break;
	}
}


bool CNodeData::IsRFCommUuid()
{
	if(type == SDP_TYPE_UUID)
	{
		switch(specificType)
		{
		case SDP_ST_UUID16:
			return u.uuid16 == RFCOMM_PROTOCOL_UUID16;

		case SDP_ST_UUID128:
			return u.uuid128 == RFCOMM_PROTOCOL_UUID;
		}
	}

	return false;
}


bool CNodeData::GetInt32(int &nData)
{
	bool bRet = false;

	if(type == SDP_TYPE_INT || type == SDP_TYPE_UINT)
	{
		bRet = true;

		switch(specificType)
		{
		case SDP_ST_UINT8:
			nData = u.uint8;
			break;

		case SDP_ST_UINT16:
			nData = u.uint16;
			break;

		case SDP_ST_UINT32:
			nData = u.uint32;
			break;

		case SDP_ST_UINT64:
			nData = u.uint64;
			break;

		case SDP_ST_INT8:
			nData = u.int8;
			break;

		case SDP_ST_INT16:
			nData = u.int16;
			break;

		case SDP_ST_INT32:
			nData = u.int32;
			break;

		case SDP_ST_INT64:
			nData = u.int64;
			break;

		default:
			bRet = false;
		}
	}

	return bRet;
}


bool CNodeData::GetUInt32(ULONG &nData)
{
	bool bRet = false;

	if(type == SDP_TYPE_INT || type == SDP_TYPE_UINT)
	{
		bRet = true;

		switch(specificType)
		{
		case SDP_ST_UINT8:
			nData = u.uint8;
			break;

		case SDP_ST_UINT16:
			nData = u.uint16;
			break;

		case SDP_ST_UINT32:
			nData = u.uint32;
			break;

		case SDP_ST_UINT64:
			nData = u.uint64;
			break;

		case SDP_ST_INT8:
			nData = u.int8;
			break;

		case SDP_ST_INT16:
			nData = u.int16;
			break;

		case SDP_ST_INT32:
			nData = u.int32;
			break;

		case SDP_ST_INT64:
			nData = u.int64;
			break;

		default:
			bRet = false;
		}
	}

	return bRet;
}


TCHAR* CNodeData::ToString()
{
	memset(m_szFmt, 0, sizeof(m_szFmt));

	switch(type)
	{
	case SDP_TYPE_NIL:
		wcscpy(m_szFmt, L"Nil");
		break;

	case SDP_TYPE_UINT:
		switch(specificType)
		{
		case SDP_ST_UINT8:
			wsprintf(m_szFmt, L"%d", u.uint8);
			break;

		case SDP_ST_UINT16:
			wsprintf(m_szFmt, L"%hu", u.uint16);
			break;

		case SDP_ST_UINT32:
			wsprintf(m_szFmt, L"%u", u.uint32);
			break;

		case SDP_ST_UINT64:
			wsprintf(m_szFmt, L"%lu", u.uint64);
			break;

		case SDP_ST_INT128:
			wsprintf(m_szFmt, L"%lu:%lu", u.uint128.HighPart, u.uint128.LowPart);
			break;
		}
		break;

	case SDP_TYPE_INT:
		switch(specificType)
		{
		case SDP_ST_INT8:
			wsprintf(m_szFmt, L"%d", u.int8);
			break;

		case SDP_ST_INT16:
			wsprintf(m_szFmt, L"%hd", u.int16);
			break;

		case SDP_ST_INT32:
			wsprintf(m_szFmt, L"%d", u.int32);
			break;

		case SDP_ST_INT64:
			wsprintf(m_szFmt, L"%ld", u.int64);
			break;

		case SDP_ST_INT128:
			wsprintf(m_szFmt, L"%ld:%ld", u.int128.HighPart, u.int128.LowPart);
			break;
		}
		break;

	case SDP_TYPE_UUID:
		switch(specificType)
		{
		case SDP_ST_UUID16:
			wsprintf(m_szFmt, L"0x%04X", u.uuid16);
			break;

		case SDP_ST_UUID32:
			wsprintf(m_szFmt, L"0x%08X", u.uuid32);
			break;

		case SDP_ST_UUID128:
			wsprintf(m_szFmt, 
					 L"%08X-%04X-%04X-%02X%02X-%02X%02X%02X%02X%02X%02X",
					 u.uuid128.Data1, u.uuid128.Data2, u.uuid128.Data3,
					 u.uuid128.Data4[0], u.uuid128.Data4[1], u.uuid128.Data4[2],
					 u.uuid128.Data4[3], u.uuid128.Data4[4], u.uuid128.Data4[5],
					 u.uuid128.Data4[6], u.uuid128.Data4[7]);
			break;
		}
		break;

	case SDP_TYPE_STRING:
		MultiByteToWideChar(CP_ACP, 0, u.str.val, u.str.length, m_szFmt, 127);
		break;

	case SDP_TYPE_URL:
		MultiByteToWideChar(CP_ACP, 0, u.url.val, u.url.length, m_szFmt, 127);
		break;

	case SDP_TYPE_BOOLEAN:
		return u.booleanVal ? L"True" : L"False";
		break;

	case SDP_TYPE_SEQUENCE:
		wcscpy(m_szFmt, L"Sequence");
		break;

	case SDP_TYPE_ALTERNATIVE:
		wcscpy(m_szFmt, L"Alternative");
		break;

	case SDP_TYPE_CONTAINER:
		wcscpy(m_szFmt, L"Container");
		break;

	default:
		wsprintf(m_szFmt, L"%d", type);
	}

	return m_szFmt;
}
