// SdpRecord.cpp: implementation of the CSdpRecord class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SdpRecord.h"
#include "NodeData.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSdpRecord::CSdpRecord(ISdpRecord* pSdpRecord)
:	m_pSdpRecord	(pSdpRecord)
{
}


CSdpRecord::~CSdpRecord()
{
	if(m_pSdpRecord != NULL)
		m_pSdpRecord->Release();
}


HRESULT CSdpRecord::GetAttributeData(USHORT nAttr, NodeData *pNode)
{
	return m_pSdpRecord->GetAttribute(nAttr, pNode);
}


BOOL CSdpRecord::FindServiceClass(GUID *pGuid)
{
	HRESULT	hr;

	hr = m_pSdpRecord->GetServiceClass(pGuid);
	return SUCCEEDED(hr);
}


BOOL CSdpRecord::FindRFCommChannel(ULONG &nChannel)
{
	CNodeData			nodeData,
						protocolDescriptor;
	ISdpNodeContainer*	pContainer		= NULL;
	DWORD				iProtocol,
						iAtom,
						cProtocols		= 0,
						cProtocolAtoms	= 0;

	if(m_pSdpRecord->GetAttribute(SDP_ATTRIB_PROTOCOL_DESCRIPTOR_LIST, &nodeData) != ERROR_SUCCESS)
		return FALSE;

	if(nodeData.type != SDP_TYPE_CONTAINER)
		return FALSE;

	pContainer = nodeData.u.container;
	pContainer->GetNodeCount(&cProtocols);

	for(iProtocol = 0; iProtocol < cProtocols; ++iProtocol)
	{
		ISdpNodeContainer*	pProtocolContainer	= NULL;

		pContainer->GetNode(iProtocol, &protocolDescriptor);

		if(protocolDescriptor.type != SDP_TYPE_CONTAINER)
			continue;

		pProtocolContainer = protocolDescriptor.u.container;
		pProtocolContainer->GetNodeCount(&cProtocolAtoms);

		for(iAtom = 0; iAtom < cProtocolAtoms; ++iAtom)
		{
			CNodeData	nodeAtom;

			pProtocolContainer->GetNode(iAtom, &nodeAtom);

			if(nodeAtom.IsRFCommUuid())
			{
				if(iAtom + 1 != cProtocolAtoms)
				{
					CNodeData	channelID;

					pProtocolContainer->GetNode(iAtom + 1, &channelID);
					if(channelID.GetUInt32(nChannel))
						return TRUE;
				}
			}
		}
	}

	return FALSE;
}


BOOL CSdpRecord::FindServiceName(TCHAR *pszServiceName, int nNameLen)
{
	HRESULT hr;
	USHORT	nLang;
	WCHAR*	pString;

	hr = m_pSdpRecord->GetString(STRING_PROVIDER_NAME_OFFSET, &nLang, &pString);
	if(SUCCEEDED(hr))
	{
		wcscpy(pszServiceName, pString);
		CoTaskMemFree(pString);
		return TRUE;
	}

	return FALSE;
}
